c
c  program commake
c
c $Id: commake.f,v 1.3 2013/12/31 03:39:11 agnew Exp agnew $
c 
c $Log: commake.f,v $
c Revision 1.3  2013/12/31 03:39:11  agnew
c fix nonstandard usages caught by gfortran compiler
c
c Revision 1.2  2009/09/03 05:12:32  agnew
c Added checking of inputs, and default values when possible.
c
c Revision 1.1  2009/05/28 17:04:40  agnew
c Initial revision
c
c
c  Reads files of station locations and noise parameters, and generates
c    a file of common-mode noise sources for use in fakenet
c
      parameter (inmx=500)
      parameter (maxsta=1000)
      parameter (maxcom=100)
      parameter (maxcs=100)
      double precision ctep,per
c
c common block for limits of sites
c
      common/overtime/geog(4)
c
c common block for station codes, locations, and velocities
c
      common/stas/nsta,slat(maxsta),slong(maxsta),ht(maxsta)
c
c common block for common-mode noise parameters
c
      common /cmnoi/cwh(maxcom,3),cfl(maxcom,3),crw(maxcom,3),
     2  cpows(maxcom,3),cpowe(maxcom,3),nwh,nfl,nrw,npow
c
c common block for common-mode sinusoids
c
      common /cmsins/ctep(maxcs),csin(maxcs,3,2),per(maxcom),
     1  numcs
c
c common blocks of scanrp for handling commands:
c
      character*80 input(inmx)
      common /store/ input
      common /ndict/ iecho,nin
c
c Read commands; completed when reads  exec  command
c
      call scanrp
c
c Initialize random number generator, and find output name
c  
      call rangod
c
c Read station names and positions
c
      call rdsta1
c
c Read common-mode noise parameters
c
      call rdcommn
c
c Compute and write out the results
c
      call commake
c
      stop
      end
c=======================================================================
      subroutine rangod
c
c common block for random-number seed
c
      common/rseed/is
c
c initialize random-number generator
c
      call getint('seed',is,iok)
      if(iok.le.0) is=12345
      call init_genrand(is)
      return
      end
c=======================================================================
      subroutine rdsta1
c
c$$$$ calls getarr,getchr
c
c  Reads in station location information, keeping only stations that fall
c within spatial limits. File is assumed to have the format
c   code lat long height
c where the lat and long are geographic coordinates, in decimal degrees
c (East positive) and the height c is in m
c
      parameter (maxsta=1000)
      character*80 name
      character*4 ctmp
c
c common block for limits of sites
c
      common/overtime/geog(4)
c
c common block for station codes and locations
c
      common/stas/nsta,slat(maxsta),slong(maxsta),ht(maxsta)
c
      data lui/11/
      nsta=1
c
c get geographic limits
c
      call getarr('area',geog,4,nf)
      if (nf.ge.0.and.nf.lt.4) then
        write(6,100)
 100    format(' Command  area  needs Slat Nlat Wlong Elong')
        stop
      endif
      if(geog(3).gt.180) geog(3)=geog(3)-360
      if(geog(4).gt.180) geog(4)=geog(4)-360
      if(nf.lt.0) then
        geog(1)=-90
        geog(2)=90
        geog(3)=-180
        geog(4)=180
      endif
c
c get input filename, and read the data
c
      call getchr('loca',name,iok)
      if (iok.le.0) then
        write(6,102)
 102    format(' Command  location  needs name for station file') 
        stop
      endif
      open(lui,file=name,status='old')
 5    read(lui,*,end=7) ctmp,tlat,tlong
      if(tlong.gt.180) tlong=tlong-360
      if(tlat.ge.geog(1).and.tlat.le.geog(2).and.
     2 tlong.ge.geog(3).and.tlong.le.geog(4)) then
         slat(nsta)=tlat
         slong(nsta)=tlong
         ht(nsta)=tht
         nsta=nsta+1
      endif
      if(nsta.le.maxsta) go to 5
      write(6,104) maxsta
 104  format(' Only ',i6, ' station locations read in')
 7    nsta=nsta-1
      close(lui)
      return
      end
c=======================================================================
      subroutine rdcommn
c
c$$$$ calls getchr
c
c  Reads in noise information for common-mode series, and computes them
c
      parameter (maxcom=100)
      parameter (maxcs=100)
      character*80 name,line
      character*1 type,c
      double precision ctep,per
      double precision dat2jul
      dimension p(3),pe(3)
      dimension it(5),itm(6)
c
c common block for common-mode noise parameters
c
      common /cmnoi/cwh(maxcom,3),cfl(maxcom,3),crw(maxcom,3),
     2  cpows(maxcom,3),cpowe(maxcom,3),nwh,nfl,nrw,npow
c
c common block for common-mode sinusoids; ctep is the epoch time,
c csin the sine and cosine amplitudes, and per the periods, for each
c of the three components, and numcs the total number of sinusoids
c
      common /cmsins/ctep(maxcs),csin(maxcs,3,2),per(maxcom),
     1  numcs
c
      data lui/11/
      data dr/0.01745329252/
c
c get input filename, and read the data
c
      call getchr('comm',name,iok)
      if (iok.lt.0) return
      if (iok.le.0) then
        if(iok.lt.0) write(6,100)
 100    format(' Need name of input (common-mode noise) file')
        if(iok.eq.0) write(6,102)
 102    format(' Command  common  needs name for noise file') 
        stop
      endif
      nwh=0
      nfl=0
      nrw=0
      npow=0
      numcs=0
      open(lui,file=name,status='old')
 5    read(lui,104,end=7) line(1:80)
 104  format(a80)
      do k=1,3
        p(k)=0
      enddo
      c=line(1:1)
      if(c.eq."w".or.c.eq."r".or.c.eq."f") then
        read(line,*) type, (p(i),i=1,3)
        if(c.eq."w") then
          nwh=nwh+1
          do k=1,3
            cwh(nwh,k)=p(k)
          enddo
        endif
        if(c.eq."f") then
          nfl=nfl+1
          do k=1,3
           cfl(nfl,k)=p(k)
          enddo
        endif
        if(c.eq."r") then
          nrw=nrw+1
          do k=1,3
           crw(nrw,k)=p(k)
          enddo
        endif
      endif
      if(c.eq."p") then
        npow=npow+1
        read(line,*) type, (p(i),pe(i),i=1,3)
        do k=1,3
         cpows(npow,k)=p(k)
         cpowe(npow,k)=pe(k)
        enddo
      endif
c
c Read parameters for sinusoidal behavior.
c
      if(c.eq."s") then
        numcs=numcs+1
        read(line,*) type,(pe(i),i=1,3),(p(i),i=1,3),per(numcs),
     1       (it(i),i=1,5)
        call toymd(it,itm)
        ctep(numcs) = dat2jul(itm)
        do k=1,3
          ph=dr*p(k)
          csin(numcs,k,1)=pe(k)*cos(ph)
          csin(numcs,k,2)=pe(k)*sin(ph)
        enddo
      endif
      go to 5
 7    close(lui)
      return
      end
c=======================================================================
      subroutine commake
      parameter (maxsta=1000)
      parameter (maxcom=100)
      parameter (maxcs=100)
      character*80 name
      double precision ctep,per
      double precision genrand_real3
      dimension it(5),itm(6)
      dimension cp(3),cq(3),kt(maxcs),pt(maxcs)
c
c common block for station codes, locations, and velocities
c
      common/stas/nsta,slat(maxsta),slong(maxsta),ht(maxsta)
c
c common block for common-mode noise parameters
c
      common /cmnoi/cwh(maxcom,3),cfl(maxcom,3),crw(maxcom,3),
     2  cpows(maxcom,3),cpowe(maxcom,3),nwh,nfl,nrw,npow
c
c common block for common-mode sinusoids
c
      common /cmsins/ctep(maxcs),csin(maxcs,3,2),per(maxcom),
     1  numcs
c
      data luo/12/
      data dr/0.01745329252/
c 
c get the output file name
c
      call getchr('outp',name,iok)
      if(iok.le.0) then
         write(6,100)
 100     format(' Need name of output (common-mode noise) file')
         stop
      endif
      open(luo,file=name)
      call centroid(slat,slong,nsta,clat,clong)
      call getarr('dist',cp,2,iok)
      if(iok.ge.0.and.iok.lt.2) then
         write(6,104)
  104    format(' Command  distance  needs number of centers, and',
     1    ' distance')
         stop
      endif
      if(iok.lt.0) then
        nc=1
        dist=0.
      endif
      if(iok.eq.2) then
        nc=nint(cp(1))
        dist=cp(2)
      endif
      nsin=0
      call getarr('nsin',cp,1,iok)
      if(iok.eq.1) nsin=nint(cp(1))
c
c generate  nc  centers of common-mode noise
c
      do i=1,nc
        az=(360./nc)*(i-1)
        call topod(clat,clong,dist,az,celat,celong)
        write(luo,110) "l",celat,celong
 110    format(a1,1x,f9.5,f12.5)
c
c select values randomly from set read in, separately for each component,
c  for white, flicker, random-walk, and power-law noise
c
        do j=1,3
          k=int(nwh*genrand_real3())+1
          cp(j)=cwh(k,j)
        enddo
        if(cp(1).ne.0.or.cp(2).ne.0.or.cp(3).ne.0)
     1    write(luo,112) "w",(cp(l),l=1,3)
 112        format(a1,1x,3f10.4)
        do j=1,3
          k=int(nfl*genrand_real3())+1
          cp(j)=cfl(k,j)
        enddo
        if(cp(1).ne.0.or.cp(2).ne.0.or.cp(3).ne.0)
     1    write(luo,112) "f",(cp(l),l=1,3)
        do j=1,3
          k=int(nrw*genrand_real3())+1
          cp(j)=crw(k,j)
        enddo
        if(cp(1).ne.0.or.cp(2).ne.0.or.cp(3).ne.0)
     1    write(luo,112) "r",(cp(l),l=1,3)
        do j=1,3
          k=int(npow*genrand_real3())+1
          cp(j)=cpows(k,j)
          cq(j)=cpowe(k,j)
        enddo
        if(cp(1).ne.0.or.cp(2).ne.0.or.cp(3).ne.0)
     1       write(luo,114) "p",(cp(l),cq(l),l=1,3)
 114    format(a1,1x,6f10.4)
c
c write out up to nsin sinusoids, randomly selected, but do not allow
c  repeats, or more than one with the same period
c
        if(nsin.gt.0) then
          do j=1,nsin
            k=int(numcs*genrand_real3())+1
c
c check for previous use of this index, or period (with some tolerance,
c   needed because of finite precision on reading
c
            kt(j)=k
            pt(j)=per(k)
            is=0
            if(j.gt.1) then
              do l=1,j-1
                if(k.eq.kt(l).or.abs(per(k)-pt(l)).lt.0.0001) is=1
              enddo
            endif
c
c convert back to amp and phase, and write out
c 
            if(is.eq.0) then
              call jul2dat(ctep(k),itm)
              call toyd(itm,it)
              do l=1,3
                cp(l)=sqrt(csin(k,l,1)**2+csin(k,l,2)**2)
                cq(l)=0.
                if(cp(l).gt.0) then 
                  cq(l)=atan2(csin(k,l,2),csin(k,l,1))/dr
                  if(cq(l).lt.-180) cq(l)=cq(l)+360
                  if(cq(l).gt.180) cq(l)=cq(l)-360
                endif
              enddo
              write(luo,116) "s",(cp(l),l=1,3),(cq(l),l=1,3),per(k),it
 116          format(a1,1x,3f8.4,3f7.1,f10.4,1x,5i4)
            endif
          enddo
        endif
      enddo
      close(luo)
      return
      end
c=======================================================================
c ***********************************************************************
c *                                                                     *
c *   Routines below for reading and processing script commands are     *
c *     provided courtesy of R. L. Parker; specific commands in review  *
c *     listing in scanrp, and checking in icheck, have been modified   *
c *     for this code.                                                  *
c *                                                                     *
c ***********************************************************************
c=======================================================================
      subroutine scanrp
c
c$$$$ calls clear getchr icheck
c
c  Input routine for commands 
c
c  Reads from the standard input until eof or code "execute " or "quit".
c  Saves lines in the input store  /store/ for later retrieval by
c  getarr, getone or getchr.
c
c  Prints a glossary upon request
c
      parameter (inmx=500)
      character*4 code
      character*80 input(inmx),line
      common /store/ input
      common /ndict/ iecho,nin
c
      if (nin .eq. 0) write(6,'(a)') ' ',
     $'Enter commands for producing common-mode file (? for help)'
c
      do 1500 l=nin+1, inmx
        read(*,'(80a)', end=3000) line
        if (line(1:4).eq.'cont' .or. line(1:4).eq.'exec') goto 2000
c  List a glossary of codes
        if (line(1:1) .eq. '?') then
          write(6, '(a/a/(2x,a))')
     $'Enter commands from the following list:',
     $'?:        Remind me of the command list again',
     $'execute:           Quit reading commands and begin calculations',
     $'output filename:                 Enter filename for output file',
     $'common filename:           Enter filename for common-mode noise',
     $'area south north west east:           Set lat and lon to select',
     $'location filename:    Enter filename of station list (required)',
     $'nsines number:           Set maximum number of sinusoids to use',
     $'seed iseed:                Set seed for random number generator',
     $'distance distance:          Set distance to common-mode centers'
          write(6, '(2x,a)')
     $'review:                           Display current command stack',
     $'clear command:     Delete most recent occurrence of the command',
     $' '
c
c  Review the command stack
        elseif (line(1:4) .eq. 'revi') then
          write(6,'(5x,a)')' ', '=================== ',
     $    (input(i)(1:60),i=1,nin),'=================== '
c
        elseif (line(1:1) .ne. ' ') then
          if (line(1:4) .eq. 'echo') iecho=-iecho
          if (line(1:4) .eq. 'quit') then
            write(6,'(/a)') 'Normal termination'
            stop
          endif
          nin=nin + 1
          if (nin .gt. inmx) then
            write(6,'(a)') '>>>> Too many commands - memory full'
            stop
          endif
          call icheck(line)
          input(nin)=line
c  Clear a command and clear the clear command
          if (line(1:4) .eq. 'clea') then
            call getchr('clear', code, klear)
            call clear (code)
            nin=nin - 1
          endif
        endif
 1500 continue
c
 2000 continue
      n1=max(1, nin-24)
      write(6,'(5x,a)')' ', '=================== ',
     $(input(i)(1:60),i=n1,nin),'=================== '
      return
c
 3000 stop
      end
c=======================================================================
      subroutine icheck(line)
c
c$$$$ calls no other routines
c
c  Checks the command fragment com against a list; appends a
c  warning if com is not present in the list.
      character*80 line
      character*4 com
      com=line(1:4)
      if (0.eq.
     1 index('area comm dist loca outp nsin',com)+
     2 index('seed',com))
     2 line=line(1:20)//' <<<<<<< Unrecognized command'
      return
      end
c=======================================================================
